package com.yahoo.research.robme.anthelion.models.banditfunction;

import java.util.List;
import java.util.Properties;

import com.yahoo.research.robme.anthelion.models.AnthHost;
import com.yahoo.research.robme.anthelion.models.AnthURL;
import com.yahoo.research.robme.anthelion.models.HostValueUpdateNecessity;

/**
 * Bandit Function using AbsoluteGood combined with the Best Score Function
 * (Based on selected Classifier).
 * 
 * @author Robert Meusel
 * 
 */
public class AbsoluteGoodBestScoreFunction implements DomainValueFunction {

	@Override
	public double getDomainValue(AnthHost host) {

		// initially its the minimal double value
		// as this can only happen when something is wrong with the host we want
		// to ignore it
		double out = Double.MIN_VALUE;
		AnthURL url = null;
		if ((url = host.peekNextURL()) != null) {
			try {
				out = url.prediction;
			} catch (Exception e) {
				// just do nothing.
			}
		}
		return out * host.goodUrls;

	}

	@Override
	public void setProperty(Properties prop) {
		// can be ignored
	}

	@Override
	public boolean getNecessities(HostValueUpdateNecessity nec) {
		if (nec == HostValueUpdateNecessity.ON_STAT_CHANGE){
			return true;
		}
		if (nec == HostValueUpdateNecessity.ON_QUEUE_CHANGE){
			return true;
		}
		return false;
	}

}
